@@ -1,9 +1,9 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; procmail reader for Wanderlust on Emacsen.
-;;; $Id: prom-wl.el,v 2.16 1999/06/10 12:02:14 murata Exp $
+;;; $Id: prom-wl.el,v 2.28 2000/11/11 01:47:19 murata Exp $
 ;;; by Masahiro MURATA <muse@ba2.so-net.ne.jp>
 
-(defconst prom-wl-version "Prom-WL 1.0.2")
+(defconst prom-wl-version "Prom-WL 1.1.2")
 
 ;; !!! this version is required Wanderlust 1.0.2 or later !!!
 
@@ -43,6 +43,14 @@ If file prefix is '%', log file exists on imap4 server.
 (defvar proc-lock-file "~/Mail/.lock"
   "*Global lockfile of procmail.")
 
+(defvar proc-folder-regexp nil
+  "*Optional folder regexp for checking procmail logfile.
+
+If you use dmail in ~/.procmailrc, try:
+
+(setq proc-folder-regexp \"^  Folder:[ ]*dmail[ ]+\\\\+\\\\([^ \\t]+\\\\)\")
+")
+
 (defvar prom-use-lockfile t
   "*If non-nil, use lockfile program of procmail package.")
 
@@ -95,6 +103,9 @@ If file prefix is '%', log file exists on imap4 server.
 
 (defvar prom-wl-lock-localdir-msgdb-create nil)
 
+(defvar prom-wl-set-promrc-log-func 'prom-wl-set-promrc-log
+  "*A function for parsing procmail logfile.")
+
 ;;; Hooks
 
 (defvar prom-wl-get-new-mail-pre-hook nil)
@@ -112,6 +123,7 @@ If file prefix is '%', log file exists on imap4 server.
 (defvar prom-check-entity-list nil)
 (defvar prom-lock-status nil)
 (defvar prom-wl-previous-window-config nil)
+(defvar prom-wl-compatible-imap4-get-connection nil)
 
 (defconst prom-buffer-tmp " *prom-wl tmp*")
 
@@ -119,6 +131,22 @@ If file prefix is '%', log file exists on imap4 server.
 ;;; @ code
 ;;;
 
+;; for older wl.
+(defun prom-wl-imap4-get-connection (spec)
+  (if prom-wl-compatible-imap4-get-connection
+      (elmo-imap4-get-connection (nth 3 spec)
+				 (nth 2 spec)
+				 (nth 4 spec)
+				 (nth 5 spec)
+				 (nth 6 spec))
+    (condition-case err
+	(elmo-imap4-get-connection spec)
+      (error (if (not (eq (car err) 'wrong-number-of-arguments))
+		 (signal (car err) (cdr err))
+	       (setq prom-wl-compatible-imap4-get-connection t)
+	       ;; call myself :-p
+	       (prom-wl-imap4-get-connection spec))))))
+
 (defvar prom-wl-generate-mailer-string-func nil
   "(setq prom-wl-generate-mailer-string-func wl-generate-user-agent-string)")
 
@@ -140,6 +168,11 @@ If file prefix is '%', log file exists on imap4 server.
 	 (and (get-buffer tmp-buf)
 	      (kill-buffer tmp-buf))))))
 
+(defmacro prom-wl-plugged-p (folder)
+  (` (if (fboundp 'elmo-folder-plugged-p)
+	 (elmo-folder-plugged-p (, folder))
+       wl-plugged)))
+
 ;;; for wl-folder mode
 ;;;
 
@@ -177,6 +210,18 @@ If file prefix is '%', log file exists on imap4 server.
       (message "no more unread folder")
       nil)))
 
+(defun prom-wl-folder-jump-folders (folders)
+  (let (point select-folder)
+    (save-excursion
+      (setq point
+	    (catch 'found
+	      (goto-char (point-min))
+	      (while (setq select-folder (prom-wl-folder-next-unread t))
+		(if (wl-string-member select-folder folders)
+		    (throw 'found (point)))))))
+    (if point
+	(goto-char point))))
+
 (defun prom-wl-folder-open-folder2 (folders)
   (let* (id path path-list)
     (while folders
@@ -215,9 +260,8 @@ If file prefix is '%', log file exists on imap4 server.
   (interactive "P")
   (if (not (memq major-mode '(wl-folder-mode wl-summary-mode)))
       (setq prom-wl-previous-window-config (current-window-configuration)))
-  (wl-init)
   (let ((wl-auto-check-folder-name 'none))
-    (wl t))
+    (wl))
   (let ((prom-wl-auto-select-folder
 	 (if arg
 	     nil
@@ -249,7 +293,7 @@ If file prefix is '%', log file exists on imap4 server.
 (defun prom-wl-check-new-mail (&optional arg)
   "Check New mail, from procmail log files.
 If arg is non-nil, check unread folders."
-  (let (status ufolder)
+  (let (status select-folder)
     (prom-wl-setup)
     (setq prom-check-entity nil)
     (setq prom-check-entity-list nil)
@@ -309,14 +353,11 @@ If arg is non-nil, check unread folders."
 				  'folder))))
 	    (setq entity-list (cdr entity-list))))
 	(prom-wl-folder-open-folder2 unread-folders))
-      (let ((start-point (point)))
-	(goto-char (point-min))
-	(unless (setq ufolder (prom-wl-folder-next-unread t))
-	  (goto-char start-point))))
+      (setq select-folder
+	    (prom-wl-folder-jump-folders prom-check-entity-list)))
     (if (and prom-wl-auto-select-folder
-	     ufolder)
-	(wl-folder-jump-to-current-entity))
-    ))
+	     select-folder)
+	(wl-folder-jump-to-current-entity))))
 
 ;;; get-new-mail optional-method
 
@@ -338,13 +379,14 @@ If arg is non-nil, check unread folders."
        (setq proc-log (car log-list))
        (setq type (elmo-folder-get-type proc-log))
        (cond
-	((not type)		;; local logfile
+	((or (not type)		;; local logfile
+	     (string-match "^\\(.:\\)?/" proc-log))
 	 (when (file-exists-p proc-log)
 	   (goto-char (point-max))
 	   (as-binary-input-file (insert-file-contents proc-log))
 	   (delete-file proc-log)))
 	((eq type 'imap4)	;; logfile on imap4 server
-	 (when wl-plugged
+	 (when (prom-wl-plugged-p proc-log)
 	   (let (contents)
 	     (message "checking log of imap server...")
 	     (when (setq contents
@@ -358,11 +400,10 @@ If arg is non-nil, check unread folders."
 	 (unless (setq prefix (prom-wl-get-folder-prefix type))
 	   (error "no prefix of type: %s" type))
 	 (prom-wl-append-keep-log)
-	 (prom-wl-set-promrc-log prefix)
+	 (funcall prom-wl-set-promrc-log-func prefix)
 	 (erase-buffer))
        (setq log-list (cdr log-list)))
-     (run-hooks 'prom-wl-get-proc-log-hook)
-     )))
+     (run-hooks 'prom-wl-get-proc-log-hook))))
 
 (defun prom-wl-append-keep-log ()
   (when proc-keep-log
@@ -377,7 +418,7 @@ If arg is non-nil, check unread folders."
 
 ;;; get log by imap4
 
-(defvar prom-wl-imap4-close nil)
+(defvar prom-wl-imap4-close t)
 
 (defvar prom-wl-imap4-read-old-folder t)
 
@@ -386,6 +427,8 @@ If arg is non-nil, check unread folders."
 (defmacro prom-wl-imap4-log2tmp (log)
   (` (format "%s.prom-wl" (, log))))
 
+;; for old elmo-imap4.el
+
 (defun prom-wl-imap4-filter (string)
   "*Erase a prefix of imap4."
   (elmo-replace-in-string
@@ -393,21 +436,22 @@ If arg is non-nil, check unread folders."
     string "\n?\n)" "")
    "\* 1 FETCH (BODY\[\] \{[0-9]+\}" prom-wl-dummy-from))
 
-(defun prom-wl-imap4-read-log (proc-log)
+(defun prom-wl-imap4-select-folder (folder connection)
+  (condition-case nil
+      (elmo-imap4-select-folder folder connection)
+    (error)))
+
+(defun prom-wl-imap4-read-log-oldcompat (proc-log)
   (save-excursion
     (let* ((spec (elmo-folder-get-spec proc-log))
 	   (folder (nth 1 spec))
-	   (connection (elmo-imap4-get-connection (nth 3 spec)
-						  (nth 2 spec)
-						  (nth 4 spec)
-						  (nth 5 spec)
-						  (nth 6 spec)))
+	   (connection (prom-wl-imap4-get-connection spec))
 	   (buffer (car connection))
 	   (process (cadr connection))
 	   (length 0)
 	   response)
       (when (setq response
-		  (elmo-imap4-select-folder folder connection))
+		  (prom-wl-imap4-select-folder folder connection))
 	(if (string-match "\\* \\([0-9]+\\) EXISTS" response)
 	    (setq length (string-to-int (elmo-match-string 1 response))))
 	(setq response nil)
@@ -417,7 +461,7 @@ If arg is non-nil, check unread folders."
 	  (if (null (elmo-imap4-read-response buffer process))
 	      (error "Store failed"))
 	  ;; lock
-;; 	  (if (null (elmo-imap4-select-folder folder connection))
+;; 	  (if (null (prom-wl-imap4-select-folder folder connection))
 ;; 	      (error "%s: select folder failed" folder))
 	  (elmo-imap4-send-command
 	   buffer process "fetch 1 body.peek[]")
@@ -435,28 +479,24 @@ If arg is non-nil, check unread folders."
 	      (error "close failed")))
 	response))))
 
-(defun prom-wl-imap4-read-log-by-rename (proc-log)
+(defun prom-wl-imap4-read-log-by-rename-oldcompat (proc-log)
   (save-excursion
     (let* ((spec (elmo-folder-get-spec proc-log))
 	   (folder (nth 1 spec))
 	   (temp-folder (prom-wl-imap4-log2tmp folder))
-	   (connection (elmo-imap4-get-connection (nth 3 spec)
-						  (nth 2 spec)
-						  (nth 4 spec)
-						  (nth 5 spec)
-						  (nth 6 spec)))
+	   (connection (prom-wl-imap4-get-connection spec))
 	   (buffer (car connection))
 	   (process (cadr connection))
 	   old-contents retval)
       (when prom-wl-imap4-read-old-folder
 	(setq old-contents
-	      (prom-wl-imap4-read-and-delete temp-folder connection t)))
+	      (prom-wl-imap4-read-and-delete-oldcompat temp-folder connection t)))
       (elmo-imap4-send-command
        buffer process (format "rename %s %s" folder temp-folder))
       (if (elmo-imap4-read-response buffer process)
 	  ;; rename success
 	  (progn
-	    (setq retval (prom-wl-imap4-read-and-delete temp-folder connection))
+	    (setq retval (prom-wl-imap4-read-and-delete-oldcompat temp-folder connection))
 	    (if old-contents
 		(concat old-contents retval)
 	      retval))
@@ -466,14 +506,14 @@ If arg is non-nil, check unread folders."
 	  ;;(error "%s: rename log failed" temp-folder)
 	  )))))
 
-(defun prom-wl-imap4-read-and-delete (folder connection
-					     &optional select-noerror)
+(defun prom-wl-imap4-read-and-delete-oldcompat (folder connection
+						       &optional select-noerror)
   (let ((buffer (car connection))
 	(process (cadr connection))
 	(length 0)
 	response)
     (if (null (setq response
-		    (elmo-imap4-select-folder folder connection)))
+		    (prom-wl-imap4-select-folder folder connection)))
 	(if (not select-noerror)
 	    (error "%s: select folder failed" folder))
       (if (string-match "\\* \\([0-9]+\\) EXISTS" response)
@@ -494,34 +534,131 @@ If arg is non-nil, check unread folders."
 	  (error "%s: delete log failed" folder))
       response)))
 
+;; for new elmo-imap4.el
+
+(defun prom-wl-imap4-read-log (proc-log)
+  (if (not (fboundp 'elmo-imap4-get-session))
+      (prom-wl-imap4-read-log-oldcompat proc-log)
+    (save-excursion
+      (let* ((spec (elmo-folder-get-spec proc-log))
+	     (session (elmo-imap4-get-session spec))
+	     exists response)
+	(condition-case ()
+	    (setq exists (elmo-imap4-response-value
+			  (elmo-imap4-session-select-mailbox
+			   session
+			   (elmo-imap4-spec-mailbox spec))
+			  'exists))
+	  (error nil))
+	(with-current-buffer (elmo-network-session-buffer session)
+	  (setq elmo-imap4-fetch-callback nil)
+	  (setq elmo-imap4-fetch-callback-data nil))
+	(when (and exists (> exists 0))
+	  (and
+	   (elmo-imap4-response-value
+	    (elmo-imap4-send-command-wait session "store 1 +flags (\\Deleted)")
+	    'fetch)
+	   (setq response
+		 (concat
+		  prom-wl-dummy-from "\n"
+		  (elmo-delete-cr
+		   (elmo-imap4-response-bodydetail-text
+		    (elmo-imap4-response-value-all
+		     (elmo-imap4-send-command-wait session "fetch 1 body.peek[]")
+		     'fetch)))))
+	   (elmo-imap4-response-value
+	    (elmo-imap4-send-command-wait session "expunge")
+	    'expunge)))
+	(when prom-wl-imap4-close
+	  (elmo-network-close-session session))
+	response))))
+
+(defun prom-wl-imap4-read-log-by-rename (proc-log)
+  (if (not (fboundp 'elmo-imap4-get-session))
+      (prom-wl-imap4-read-log-by-rename-oldcompat proc-log)
+    (save-excursion
+      (let* ((spec (elmo-folder-get-spec proc-log))
+	     (folder (nth 1 spec))
+	     (temp-folder (prom-wl-imap4-log2tmp folder))
+	     (session (elmo-imap4-get-session spec))
+	     response old-contents retval)
+	(when prom-wl-imap4-read-old-folder
+	  (setq old-contents
+		(prom-wl-imap4-read-and-delete temp-folder session t)))
+	(setq response
+	      (elmo-imap4-read-response
+	       session
+	       (elmo-imap4-send-command
+		session (format "rename %s %s" folder temp-folder))))
+	(if (elmo-imap4-response-ok-p response)
+	    ;; rename success
+	    (progn
+	      (setq retval (prom-wl-imap4-read-and-delete temp-folder session))
+	      (if old-contents
+		  (concat old-contents retval)
+		retval))
+	  ;; rename fail
+	  (if old-contents
+	      old-contents))))))
+
+(defun prom-wl-imap4-read-and-delete (folder session
+					     &optional select-noerror)
+  (let (exists response)
+    (condition-case ()
+	(setq exists (elmo-imap4-response-value
+		      (elmo-imap4-session-select-mailbox session folder)
+		      'exists))
+      (error nil))
+    (with-current-buffer (elmo-network-session-buffer session)
+      (setq elmo-imap4-fetch-callback nil)
+      (setq elmo-imap4-fetch-callback-data nil))
+    (when (and exists (> exists 0))
+      (and
+       (elmo-imap4-response-value
+	(elmo-imap4-send-command-wait session "store 1 +flags (\\Deleted)")
+	'fetch)
+       (setq response
+	     (concat
+	      prom-wl-dummy-from "\n"
+	      (elmo-delete-cr
+	       (elmo-imap4-response-bodydetail-text
+		(elmo-imap4-response-value-all
+		 (elmo-imap4-send-command-wait session "fetch 1 body.peek[]")
+		 'fetch)))))
+       (elmo-imap4-send-command-wait session "close")
+       (elmo-imap4-send-command-wait session (concat "delete " folder))))
+    response))
+
 ;;; end of imap4
 
 (defun prom-wl-set-promrc-log (prefix)
   (save-excursion
     (goto-char (point-min))
-    (while (re-search-forward "^  Folder: \\(.+\\)" nil t)
-      (let ((folder-body (elmo-match-buffer 1))
-	    folder)
-	(if (string-match "^\\(.+\\)/\\([0-9]+\\)" folder-body)
-	    (let ((path (elmo-match-string 1 folder-body)))
-	      (cond ((string-match "^[%-+=]" path)
-		     (setq folder path)
-		     )
-		    ;; Absolute path (include DriveLetter)
-		    ((string-match "^\\(.:\\)?/" path)
-		     (cond
-		      ((string-match (concat "^" (expand-file-name elmo-localdir-folder-path) "/*") path)
-		       (setq folder
-			     (concat prefix (substring path (match-end 0)))) )
-		      (t
-		       (setq folder path)) ))
-		    ;; mail folder
-		    (t
-		     (setq folder (concat prefix path))
-		     ))
-	      (prom-wl-add-check-entity-list folder)
-	      )))
-      )))
+    (let (folder path)
+      (while
+	  (cond
+	   ((and proc-folder-regexp
+		 (re-search-forward proc-folder-regexp nil t)
+		 (setq folder (concat prefix (elmo-match-buffer 1))))
+	    t)
+	   ((and (re-search-forward "^  Folder: \\(.+\\)/\\([0-9]+\\)" nil t)
+		 (setq path (elmo-match-buffer 1))
+		 (progn
+		   (cond ((string-match "^[%-+=]" path)
+			  (setq folder path))
+			 ;; Absolute path (include DriveLetter)
+			 ((string-match "^\\(.:\\)?/" path)
+			  (cond
+			   ((string-match (concat "^" (expand-file-name elmo-localdir-folder-path) "/*") path)
+			    (setq folder
+				  (concat prefix (substring path (match-end 0)))))
+			   (t
+			    (setq folder path))))
+			 ;; mail folder
+			 (t
+			  (setq folder (concat prefix path))))))
+	    t))
+	(prom-wl-add-check-entity-list folder)))))
 
 (defun prom-wl-add-check-entity-list (folder)
   (if (not (member folder prom-check-entity-list))
